"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AddressMapper = void 0;
const tslib_1 = require("tslib");
const contract_1 = require("../contract");
const address_1 = require("../address");
const address_mapper_pb_1 = require("../proto/address_mapper_pb");
const solidity_helpers_1 = require("../solidity-helpers");
class AddressMapper extends contract_1.Contract {
    static createAsync(client, callerAddr) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const contractAddr = yield client.getContractAddressAsync('addressmapper');
            if (!contractAddr) {
                throw Error('Failed to resolve contract address');
            }
            return new AddressMapper({ contractAddr, callerAddr, client });
        });
    }
    constructor(params) {
        super(params);
    }
    addIdentityMappingAsync(from, to, ethersSigner) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const mappingIdentityRequest = new address_mapper_pb_1.AddressMapperAddIdentityMappingRequest();
            mappingIdentityRequest.setFrom(from.MarshalPB());
            mappingIdentityRequest.setTo(to.MarshalPB());
            const hash = (0, solidity_helpers_1.soliditySha3)({
                type: 'address',
                value: from.local.toString().slice(2)
            }, {
                type: 'address',
                value: to.local.toString().slice(2)
            });
            const sign = yield ethersSigner.signAsync(hash);
            mappingIdentityRequest.setSignature(sign);
            return this.callAsync('AddIdentityMapping', mappingIdentityRequest);
        });
    }
    /**
     * Similar to addIdentityMappingAsync but for Binance
     */
    addBinanceIdentityMappingAsync(from, to, ethersSigner) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const mappingIdentityRequest = new address_mapper_pb_1.AddressMapperAddIdentityMappingRequest();
            mappingIdentityRequest.setFrom(from.MarshalPB());
            mappingIdentityRequest.setTo(to.MarshalPB());
            const msg = from.local.toString().slice(2) + to.local.toString().slice(2);
            const sign = yield ethersSigner.signAsync('0x' + msg);
            mappingIdentityRequest.setSignature(sign);
            return this.callAsync('AddIdentityMapping', mappingIdentityRequest);
        });
    }
    hasMappingAsync(from) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const hasMappingRequest = new address_mapper_pb_1.AddressMapperHasMappingRequest();
            hasMappingRequest.setFrom(from.MarshalPB());
            const result = yield this.staticCallAsync('HasMapping', hasMappingRequest, new address_mapper_pb_1.AddressMapperHasMappingResponse());
            return result.getHasMapping();
        });
    }
    getMappingAsync(from) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const getMappingRequest = new address_mapper_pb_1.AddressMapperGetMappingRequest();
            getMappingRequest.setFrom(from.MarshalPB());
            const result = yield this.staticCallAsync('GetMapping', getMappingRequest, new address_mapper_pb_1.AddressMapperGetMappingResponse());
            return {
                from: address_1.Address.UnmarshalPB(result.getFrom()),
                to: address_1.Address.UnmarshalPB(result.getTo())
            };
        });
    }
}
exports.AddressMapper = AddressMapper;
//# sourceMappingURL=address-mapper.js.map