"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BscTransferGateway = void 0;
const tslib_1 = require("tslib");
const transfer_gateway_1 = require("./transfer-gateway");
const transfer_gateway_pb_1 = require("../proto/transfer_gateway_pb");
const big_uint_1 = require("../big-uint");
class BscTransferGateway extends transfer_gateway_1.TransferGateway {
    static createAsync(client, callerAddr) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const contractAddr = yield client.getContractAddressAsync('bsc-gateway');
            if (!contractAddr) {
                throw Error('Failed to resolve contract address for BscTransferGateway');
            }
            return new BscTransferGateway({ contractAddr, callerAddr, client });
        });
    }
    /**
     * Sends a request to the DAppChain Gateway to begin withdrawal of LOOM from the current
     * DAppChain account to an Ethereum account.
     * @param amount Amount to withdraw.
     * @param mainnetLoomcoinAddress Ethereum address of LOOM coin contract.
     * @param recipient Ethereum address of the account the token should be withdrawn to, if this is
     *                  omitted the Gateway will attempt to use the Address Mapper to retrieve the
     *                  address of the Ethereum account mapped to the current DAppChain account.
     * @returns A promise that will be resolved when the DAppChain Gateway has accepted the withdrawal
     *          request.
     */
    withdrawLoomCoinAsync(amount, mainnetLoomcoinAddress, recipient) {
        const req = new transfer_gateway_pb_1.TransferGatewayWithdrawLoomCoinRequest();
        req.setAmount((0, big_uint_1.marshalBigUIntPB)(amount));
        req.setTokenContract(mainnetLoomcoinAddress.MarshalPB());
        if (recipient) {
            req.setRecipient(recipient.MarshalPB());
        }
        return this.callAsync('WithdrawLoomCoin', req);
    }
}
exports.BscTransferGateway = BscTransferGateway;
//# sourceMappingURL=bsc-gateway.js.map