"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Coin = void 0;
const tslib_1 = require("tslib");
const contract_1 = require("../contract");
const coin_pb_1 = require("../proto/coin_pb");
const big_uint_1 = require("../big-uint");
class Coin extends contract_1.Contract {
    static createAsync(client, callerAddr) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const contractAddr = yield client.getContractAddressAsync('coin');
            if (!contractAddr) {
                throw Error('Failed to resolve contract address');
            }
            return new Coin({ contractAddr, callerAddr, client });
        });
    }
    constructor(params) {
        super(params);
    }
    getTotalSupplyAsync() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const totalSupplyReq = new coin_pb_1.TotalSupplyRequest();
            const result = yield this.staticCallAsync('TotalSupply', totalSupplyReq, new coin_pb_1.TotalSupplyResponse());
            return (0, big_uint_1.unmarshalBigUIntPB)(result.getTotalSupply());
        });
    }
    getBalanceOfAsync(owner) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const balanceOfReq = new coin_pb_1.BalanceOfRequest();
            balanceOfReq.setOwner(owner.MarshalPB());
            const result = yield this.staticCallAsync('BalanceOf', balanceOfReq, new coin_pb_1.BalanceOfResponse());
            return (0, big_uint_1.unmarshalBigUIntPB)(result.getBalance());
        });
    }
    getAllowanceAsync(owner, spender) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const allowanceReq = new coin_pb_1.AllowanceRequest();
            allowanceReq.setOwner(owner.MarshalPB());
            allowanceReq.setSpender(spender.MarshalPB());
            const result = yield this.staticCallAsync('Allowance', allowanceReq, new coin_pb_1.AllowanceResponse());
            return (0, big_uint_1.unmarshalBigUIntPB)(result.getAmount());
        });
    }
    approveAsync(spender, amount) {
        const approveReq = new coin_pb_1.ApproveRequest();
        approveReq.setSpender(spender.MarshalPB());
        approveReq.setAmount((0, big_uint_1.marshalBigUIntPB)(amount));
        return this.callAsync('Approve', approveReq);
    }
    transferAsync(to, amount) {
        const transferReq = new coin_pb_1.TransferRequest();
        transferReq.setTo(to.MarshalPB());
        transferReq.setAmount((0, big_uint_1.marshalBigUIntPB)(amount));
        return this.callAsync('Transfer', transferReq);
    }
    transferFromAsync(from, to, amount) {
        const transferFromReq = new coin_pb_1.TransferFromRequest();
        transferFromReq.setFrom(from.MarshalPB());
        transferFromReq.setTo(to.MarshalPB());
        transferFromReq.setAmount((0, big_uint_1.marshalBigUIntPB)(amount));
        return this.callAsync('TransferFrom', transferFromReq);
    }
}
exports.Coin = Coin;
//# sourceMappingURL=coin.js.map