"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DPOS = void 0;
const tslib_1 = require("tslib");
const bn_js_1 = tslib_1.__importDefault(require("bn.js"));
const contract_1 = require("../contract");
const address_1 = require("../address");
const dpos_pb_1 = require("../proto/dpos_pb");
class DPOS extends contract_1.Contract {
    static createAsync(client, callerAddr) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const contractAddr = yield client.getContractAddressAsync('dpos');
            if (!contractAddr) {
                throw Error('Failed to resolve contract address');
            }
            return new DPOS({ contractAddr, callerAddr, client });
        });
    }
    constructor(params) {
        super(params);
    }
    registerCandidateAsync(pubKey) {
        const registerCandidateReq = new dpos_pb_1.RegisterCandidateRequest();
        registerCandidateReq.setPubKey(pubKey);
        return this.callAsync('RegisterCandidate', registerCandidateReq);
    }
    unregisterCandidateAsync() {
        const unregisterCandidateReq = new dpos_pb_1.UnregisterCandidateRequest();
        return this.callAsync('UnregisterCandidate', unregisterCandidateReq);
    }
    voteAsync(candidateAddress, amount) {
        const voteReq = new dpos_pb_1.VoteRequest();
        voteReq.setCandidateAddress(candidateAddress.MarshalPB());
        voteReq.setAmount(amount);
        return this.callAsync('Vote', voteReq);
    }
    electAsync() {
        const electReq = new dpos_pb_1.ElectRequest();
        return this.callAsync('Elect', electReq);
    }
    getCandidatesAsync() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const listCandidatesReq = new dpos_pb_1.ListCandidateRequest();
            const result = yield this.staticCallAsync('ListCandidates', listCandidatesReq, new dpos_pb_1.ListCandidateResponse());
            return result.getCandidatesList().map((canditate) => {
                return {
                    address: address_1.Address.UnmarshalPB(canditate.getAddress()),
                    pubKey: canditate.getPubKey_asU8()
                };
            });
        });
    }
    getWitnessesAsync() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const listWitnessesReq = new dpos_pb_1.ListWitnessesRequest();
            const result = yield this.staticCallAsync('ListWitnesses', listWitnessesReq, new dpos_pb_1.ListWitnessesResponse());
            return result.getWitnessesList().map((witness) => {
                return {
                    pubKey: witness.getPubKey_asU8(),
                    voteTotal: new bn_js_1.default(witness.getPowerTotal()),
                    powerTotal: new bn_js_1.default(witness.getPowerTotal())
                };
            });
        });
    }
}
exports.DPOS = DPOS;
//# sourceMappingURL=dpos.js.map