"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DPOS2 = exports.LockTimeTier = exports.DelegationState = void 0;
const tslib_1 = require("tslib");
const bn_js_1 = tslib_1.__importDefault(require("bn.js"));
const contract_1 = require("../contract");
const address_1 = require("../address");
const dposv2_pb_1 = require("../proto/dposv2_pb");
const big_uint_1 = require("../big-uint");
var DelegationState;
(function (DelegationState) {
    DelegationState[DelegationState["BONDING"] = 0] = "BONDING";
    DelegationState[DelegationState["BONDED"] = 1] = "BONDED";
    DelegationState[DelegationState["UNBONDING"] = 2] = "UNBONDING";
    DelegationState[DelegationState["REDELEGATING"] = 3] = "REDELEGATING";
})(DelegationState = exports.DelegationState || (exports.DelegationState = {}));
var LockTimeTier;
(function (LockTimeTier) {
    LockTimeTier[LockTimeTier["Tier0"] = 0] = "Tier0";
    LockTimeTier[LockTimeTier["Tier1"] = 1] = "Tier1";
    LockTimeTier[LockTimeTier["Tier2"] = 2] = "Tier2";
    LockTimeTier[LockTimeTier["Tier3"] = 3] = "Tier3";
})(LockTimeTier = exports.LockTimeTier || (exports.LockTimeTier = {}));
class DPOS2 extends contract_1.Contract {
    static createAsync(client, callerAddr) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const contractAddr = yield client.getContractAddressAsync('dposV2');
            if (!contractAddr) {
                throw Error('Failed to resolve contract address');
            }
            return new DPOS2({ contractAddr, callerAddr, client });
        });
    }
    constructor(params) {
        super(params);
    }
    getTimeUntilElectionAsync() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const timeUntilElectionRequest = new dposv2_pb_1.TimeUntilElectionRequest();
            const result = yield this.staticCallAsync('TimeUntilElection', timeUntilElectionRequest, new dposv2_pb_1.TimeUntilElectionResponse());
            return new bn_js_1.default(result.getTimeUntilElection());
        });
    }
    getCandidatesAsync() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const listCandidatesReq = new dposv2_pb_1.ListCandidateRequestV2();
            const result = yield this.staticCallAsync('ListCandidates', listCandidatesReq, new dposv2_pb_1.ListCandidateResponseV2());
            return result.getCandidatesList().map((candidate) => ({
                pubKey: candidate.getPubKey_asU8(),
                address: address_1.Address.UnmarshalPB(candidate.getAddress()),
                fee: new bn_js_1.default(candidate.getFee()),
                newFee: new bn_js_1.default(candidate.getNewfee()),
                feeDelayCounter: new bn_js_1.default(candidate.getFeedelaycounter()),
                name: candidate.getName(),
                description: candidate.getDescription(),
                website: candidate.getWebsite()
            }));
        });
    }
    getValidatorsAsync() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const listValidatorReq = new dposv2_pb_1.ListValidatorsRequestV2();
            const result = yield this.staticCallAsync('ListValidators', listValidatorReq, new dposv2_pb_1.ListValidatorsResponseV2());
            return result.getStatisticsList().map((validator) => ({
                address: address_1.Address.UnmarshalPB(validator.getAddress()),
                pubKey: validator.getPubKey_asU8(),
                upblockCount: validator.getUpblockCount(),
                blockCount: validator.getBlockCount(),
                whitelistAmount: validator.getWhitelistAmount()
                    ? (0, big_uint_1.unmarshalBigUIntPB)(validator.getWhitelistAmount())
                    : new bn_js_1.default(0),
                whitelistLockTime: new bn_js_1.default(validator.getWhitelistLocktime()),
                slashPct: validator.getSlashPercentage()
                    ? (0, big_uint_1.unmarshalBigUIntPB)(validator.getSlashPercentage())
                    : new bn_js_1.default(0),
                distributionTotal: validator.getDistributionTotal()
                    ? (0, big_uint_1.unmarshalBigUIntPB)(validator.getDistributionTotal())
                    : new bn_js_1.default(0),
                delegationTotal: validator.getDelegationTotal()
                    ? (0, big_uint_1.unmarshalBigUIntPB)(validator.getDelegationTotal())
                    : new bn_js_1.default(0)
            }));
        });
    }
    getDelegations(candidate) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const listDelegationsReq = new dposv2_pb_1.ListDelegationsRequest();
            listDelegationsReq.setCandidate(candidate.MarshalPB());
            const result = yield this.staticCallAsync('ListDelegations', listDelegationsReq, new dposv2_pb_1.ListDelegationsResponse());
            return {
                delegationTotal: result.getDelegationTotal()
                    ? (0, big_uint_1.unmarshalBigUIntPB)(result.getDelegationTotal())
                    : new bn_js_1.default(0),
                delegationsArray: result.getDelegationsList().map(this.getDelegation)
            };
        });
    }
    getAllDelegations() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const listAllDelegationsReq = new dposv2_pb_1.ListAllDelegationsRequest();
            const result = yield this.staticCallAsync('ListAllDelegations', listAllDelegationsReq, new dposv2_pb_1.ListAllDelegationsResponse());
            return result.getListresponsesList().map(d => {
                return {
                    delegationTotal: d.getDelegationTotal()
                        ? (0, big_uint_1.unmarshalBigUIntPB)(d.getDelegationTotal())
                        : new bn_js_1.default(0),
                    delegationsArray: d.getDelegationsList().map(this.getDelegation)
                };
            });
        });
    }
    checkDelegatorDelegations(delegator) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const checkDelegatorDelegationsReq = new dposv2_pb_1.CheckAllDelegationsRequest();
            checkDelegatorDelegationsReq.setDelegatorAddress(delegator.MarshalPB());
            const result = yield this.staticCallAsync('CheckAllDelegations', checkDelegatorDelegationsReq, new dposv2_pb_1.CheckAllDelegationsResponse());
            return {
                amount: result.getAmount() ? (0, big_uint_1.unmarshalBigUIntPB)(result.getAmount()) : new bn_js_1.default(0),
                weightedAmount: result.getWeightedAmount()
                    ? (0, big_uint_1.unmarshalBigUIntPB)(result.getWeightedAmount())
                    : new bn_js_1.default(0),
                delegationsArray: result.getDelegationsList().map(this.getDelegation)
            };
        });
    }
    checkDistributionAsync(owner) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const checkDistributionReq = new dposv2_pb_1.CheckDistributionRequest();
            checkDistributionReq.setAddress(owner.MarshalPB());
            const result = yield this.staticCallAsync('CheckDistribution', checkDistributionReq, new dposv2_pb_1.CheckDistributionResponse());
            return result.getAmount() ? (0, big_uint_1.unmarshalBigUIntPB)(result.getAmount()) : new bn_js_1.default(0);
        });
    }
    totalDelegationAsync(delegator) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const totalDelegationReq = new dposv2_pb_1.TotalDelegationRequest();
            totalDelegationReq.setDelegatorAddress(delegator.MarshalPB());
            const result = yield this.staticCallAsync('TotalDelegation', totalDelegationReq, new dposv2_pb_1.TotalDelegationResponse());
            return {
                amount: result.getAmount() ? (0, big_uint_1.unmarshalBigUIntPB)(result.getAmount()) : new bn_js_1.default(0),
                weightedAmount: result.getWeightedAmount()
                    ? (0, big_uint_1.unmarshalBigUIntPB)(result.getWeightedAmount())
                    : new bn_js_1.default(0)
            };
        });
    }
    checkDelegationAsync(validator, delegator) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const checkDelegationReq = new dposv2_pb_1.CheckDelegationRequestV2();
            checkDelegationReq.setValidatorAddress(validator.MarshalPB());
            checkDelegationReq.setDelegatorAddress(delegator.MarshalPB());
            const result = yield this.staticCallAsync('CheckDelegation', checkDelegationReq, new dposv2_pb_1.CheckDelegationResponseV2());
            const delegation = result.getDelegation();
            return delegation ? this.getDelegation(delegation) : null;
        });
    }
    claimDistributionAsync(withdrawalAddress) {
        const claimDistributionRequest = new dposv2_pb_1.ClaimDistributionRequestV2();
        claimDistributionRequest.setWithdrawalAddress(withdrawalAddress.MarshalPB());
        return this.callAsync('ClaimDistribution', claimDistributionRequest);
    }
    registerCandidateAsync(pubKey, fee, name, description, website, tier) {
        const registerCandidateRequest = new dposv2_pb_1.RegisterCandidateRequestV2();
        registerCandidateRequest.setPubKey(pubKey);
        registerCandidateRequest.setFee(fee.toString(10));
        registerCandidateRequest.setName(name);
        registerCandidateRequest.setDescription(description);
        registerCandidateRequest.setWebsite(website);
        registerCandidateRequest.setLocktimeTier(tier);
        return this.callAsync('RegisterCandidate', registerCandidateRequest);
    }
    unregisterCandidateAsync() {
        const unregisterCandidateRequest = new dposv2_pb_1.UnregisterCandidateRequestV2();
        return this.callAsync('UnregisterCandidate', unregisterCandidateRequest);
    }
    delegateAsync(validator, amount, tier, referrer) {
        const delegateRequest = new dposv2_pb_1.DelegateRequestV2();
        delegateRequest.setValidatorAddress(validator.MarshalPB());
        delegateRequest.setAmount((0, big_uint_1.marshalBigUIntPB)(amount));
        delegateRequest.setLocktimeTier(tier);
        if (referrer) {
            delegateRequest.setReferrer(referrer);
        }
        return this.callAsync('Delegate2', delegateRequest);
    }
    redelegateAsync(oldValidator, validator, amount, referrer) {
        const redelegateRequest = new dposv2_pb_1.RedelegateRequestV2();
        redelegateRequest.setFormerValidatorAddress(oldValidator.MarshalPB());
        redelegateRequest.setValidatorAddress(validator.MarshalPB());
        redelegateRequest.setAmount((0, big_uint_1.marshalBigUIntPB)(amount));
        if (referrer) {
            redelegateRequest.setReferrer(referrer);
        }
        return this.callAsync('Redelegate', redelegateRequest);
    }
    unbondAsync(validator, amount) {
        const unbondRequest = new dposv2_pb_1.UnbondRequestV2();
        unbondRequest.setValidatorAddress(validator.MarshalPB());
        unbondRequest.setAmount((0, big_uint_1.marshalBigUIntPB)(new bn_js_1.default(amount)));
        return this.callAsync('Unbond', unbondRequest);
    }
    getDelegation(delegation) {
        return {
            validator: address_1.Address.UnmarshalPB(delegation.getValidator()),
            updateValidator: delegation.getUpdateValidator()
                ? address_1.Address.UnmarshalPB(delegation.getUpdateValidator())
                : undefined,
            delegator: address_1.Address.UnmarshalPB(delegation.getDelegator()),
            amount: delegation.getAmount() ? (0, big_uint_1.unmarshalBigUIntPB)(delegation.getAmount()) : new bn_js_1.default(0),
            updateAmount: delegation.getUpdateAmount()
                ? (0, big_uint_1.unmarshalBigUIntPB)(delegation.getUpdateAmount())
                : new bn_js_1.default(0),
            height: new bn_js_1.default(delegation.getHeight()),
            lockTime: delegation.getLockTime(),
            lockTimeTier: delegation.getLocktimeTier(),
            state: delegation.getState()
        };
    }
}
exports.DPOS2 = DPOS2;
//# sourceMappingURL=dpos2.js.map