"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrossChainUser = void 0;
const tslib_1 = require("tslib");
const debug_1 = tslib_1.__importDefault(require("debug"));
const ethers_1 = require("ethers");
const helpers_1 = require("./helpers");
const _1 = require(".");
const address_mapper_1 = require("./contracts/address-mapper");
const solidity_helpers_1 = require("./solidity-helpers");
const log = (0, debug_1.default)('crosschain-user');
/**
 * @deprecated Will be removed in loom-js v2.0.0
 */
class CrossChainUser {
    static createOfflineCrossChainUserAsync(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const provider = new ethers_1.ethers.providers.JsonRpcProvider(params.ethEndpoint);
            const wallet = new ethers_1.ethers.Wallet(params.ethereumPrivateKey, provider);
            return CrossChainUser.createCrossChainUserAsync(Object.assign({ wallet }, params));
        });
    }
    static createMetamaskCrossChainUserAsync(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const provider = new ethers_1.ethers.providers.Web3Provider(params.web3.currentProvider);
            const wallet = provider.getSigner();
            return CrossChainUser.createCrossChainUserAsync(Object.assign({ wallet }, params));
        });
    }
    static createEthSignMetamaskCrossChainUserAsync(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const wallet = (0, solidity_helpers_1.getMetamaskSigner)(params.web3.currentProvider);
            const { client, callerAddress } = yield (0, helpers_1.createDefaultEthSignClientAsync)(params.dappchainEndpoint, params.chainId, wallet);
            const ethAddress = callerAddress.local.toString();
            const mapper = yield address_mapper_1.AddressMapper.createAsync(client, callerAddress);
            const mapping = yield mapper.getMappingAsync(callerAddress);
            const loomAddress = mapping.to;
            return new CrossChainUser({ wallet, client, loomAddress, ethAddress });
        });
    }
    static createCrossChainUserAsync(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { client, publicKey } = (0, helpers_1.createDefaultClient)(params.dappchainPrivateKey, params.dappchainEndpoint, params.chainId);
            const loomAddress = new _1.Address(client.chainId, _1.LocalAddress.fromPublicKey(publicKey));
            const ethAddress = yield params.wallet.getAddress();
            const addressMapper = yield address_mapper_1.AddressMapper.createAsync(client, loomAddress);
            return new CrossChainUser({
                wallet: params.wallet,
                client,
                loomAddress,
                ethAddress,
                addressMapper
            });
        });
    }
    constructor(params) {
        this._wallet = params.wallet;
        this._loomAddress = params.loomAddress;
        this._ethAddress = params.ethAddress;
        this._client = params.client;
        this._addressMapper = params.addressMapper;
    }
    get client() {
        return this._client;
    }
    get wallet() {
        return this._wallet;
    }
    get addressMapper() {
        return this._addressMapper;
    }
    get ethAddress() {
        return this._ethAddress;
    }
    get loomAddress() {
        return this._loomAddress;
    }
    disconnect() {
        this.client.disconnect();
    }
    /**
     * Maps the user's ETH address to their DAppChain address. This MUST be called before any interaction with the gateways.
     *
     * @param account The user's account object
     * @param wallet The User's ethers wallet
     */
    mapAccountsAsync() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._addressMapper === null) {
                throw new Error(`Tried to map accounts with nil address mapper`);
            }
            const walletAddress = yield this._wallet.getAddress();
            const ethereumAddress = _1.Address.fromString(`eth:${walletAddress}`);
            if (yield this._addressMapper.hasMappingAsync(this._loomAddress)) {
                log(`${this._loomAddress.toString()} is already mapped`);
                return;
            }
            const signer = new _1.EthersSigner(this._wallet);
            yield this._addressMapper.addIdentityMappingAsync(this._loomAddress, ethereumAddress, signer);
            log(`Mapped ${this._loomAddress} to ${ethereumAddress}`);
        });
    }
}
exports.CrossChainUser = CrossChainUser;
//# sourceMappingURL=crosschain-user.js.map