"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DPOSUserV3 = void 0;
const tslib_1 = require("tslib");
const debug_1 = tslib_1.__importDefault(require("debug"));
const ethers_1 = require("ethers");
const _1 = require(".");
const contracts_1 = require("./contracts");
const gateway_user_1 = require("./gateway-user");
const solidity_helpers_1 = require("./solidity-helpers");
const log = (0, debug_1.default)('dpos3-user');
/**
 * @deprecated Will be removed in loom-js v2.0.0
 */
class DPOSUserV3 extends gateway_user_1.GatewayUser {
    static createOfflineUserAsync(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const provider = new ethers_1.ethers.providers.JsonRpcProvider(params.ethEndpoint);
            const wallet = new ethers_1.ethers.Wallet(params.ethereumPrivateKey, provider);
            return DPOSUserV3.createUserAsync(Object.assign({ wallet }, params));
        });
    }
    static createMetamaskUserAsync(params) {
        const wallet = (0, solidity_helpers_1.getMetamaskSigner)(params.web3.currentProvider);
        return DPOSUserV3.createUserAsync(Object.assign({ wallet }, params));
    }
    static createEthSignMetamaskUserAsync(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const gatewayUser = yield gateway_user_1.GatewayUser.createEthSignMetamaskGatewayUserAsync(params);
            const dappchainEthAddress = new _1.Address('eth', _1.LocalAddress.fromHexString(gatewayUser.ethAddress));
            const dappchainDPOS = yield contracts_1.DPOS3.createAsync(gatewayUser.client, dappchainEthAddress);
            log('Connected to dappchain DPOS Contract');
            return new DPOSUserV3({
                wallet: gatewayUser.wallet,
                client: gatewayUser.client,
                loomAddress: gatewayUser.loomAddress,
                ethAddress: gatewayUser.ethAddress,
                gateway: gatewayUser.ethereumGateway,
                loomToken: gatewayUser.ethereumLoom,
                vmc: gatewayUser.ethereumVMC,
                dappchainGateway: gatewayUser.dappchainGateway,
                dappchainLoom: gatewayUser.dappchainLoom,
                dappchainDPOS,
                version: params.version
            });
        });
    }
    static createUserAsync(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const gatewayUser = yield gateway_user_1.GatewayUser.createGatewayUserAsync(params);
            const dappchainEthAddress = new _1.Address('eth', _1.LocalAddress.fromHexString(gatewayUser.ethAddress));
            const dappchainDPOS = yield contracts_1.DPOS3.createAsync(gatewayUser.client, dappchainEthAddress);
            log('Connected to dappchain DPOS Contract');
            return new DPOSUserV3({
                wallet: gatewayUser.wallet,
                client: gatewayUser.client,
                loomAddress: gatewayUser.loomAddress,
                ethAddress: gatewayUser.ethAddress,
                gateway: gatewayUser.ethereumGateway,
                loomToken: gatewayUser.ethereumLoom,
                vmc: gatewayUser.ethereumVMC,
                dappchainGateway: gatewayUser.dappchainGateway,
                dappchainLoom: gatewayUser.dappchainLoom,
                addressMapper: gatewayUser.addressMapper,
                dappchainDPOS,
                version: params.version
            });
        });
    }
    constructor(params) {
        super(params);
        this._dappchainDPOS = params.dappchainDPOS;
    }
    get dappchainDPOS() {
        return this._dappchainDPOS;
    }
    listValidatorsAsync() {
        return this._dappchainDPOS.getValidatorsAsync();
    }
    listCandidatesAsync() {
        return this._dappchainDPOS.getCandidatesAsync();
    }
    listAllDelegationsAsync() {
        return this._dappchainDPOS.getAllDelegations();
    }
    listDelegationsAsync(candidate) {
        const address = this.prefixAddress(candidate);
        return this._dappchainDPOS.getDelegations(address);
    }
    checkAllDelegationsAsync(delegator) {
        const address = delegator ? this.prefixAddress(delegator) : this.loomAddress;
        return this._dappchainDPOS.checkAllDelegationsAsync(address);
    }
    getTimeUntilElectionsAsync() {
        return this._dappchainDPOS.getTimeUntilElectionAsync();
    }
    /**
     * Delegates an amount of LOOM tokens to a candidate/validator
     *
     * @param candidate The candidate's hex address
     * @param amount The amount delegated
     */
    delegateAsync(candidate, amount, tier, referrer) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const address = this.prefixAddress(candidate);
            yield this.dappchainLoom.approveAsync(this._dappchainDPOS.address, amount);
            return this._dappchainDPOS.delegateAsync(address, amount, tier, referrer);
        });
    }
    /**
     * Redelegates an amount of LOOM tokens from a validator to another
     *
     * @param formerValidator The candidate's hex address
     * @param newValidator The candidate's hex address
     * @param amount The amount delegated
     */
    redelegateAsync(formerValidator, validator, amount, index) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const validatorAddress = this.prefixAddress(validator);
            const formerValidatorAddress = this.prefixAddress(formerValidator);
            return this._dappchainDPOS.redelegateAsync(formerValidatorAddress, validatorAddress, amount, index);
        });
    }
    /**
     * Undelegates an amount of LOOM tokens from a candidate/validator
     *
     * @param candidate The candidate's hex address
     * @param amount The amount to undelegate
     */
    undelegateAsync(candidate, amount, index) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const address = this.prefixAddress(candidate);
            yield this._dappchainDPOS.unbondAsync(address, amount, index);
        });
    }
    /**
     * Returns the stake a delegator has delegated to a candidate/validator
     *
     * @param validator The validator's hex address
     * @param delegator The delegator's hex address
     */
    checkDelegationsAsync(validator, delegator) {
        const validatorAddress = this.prefixAddress(validator);
        const delegatorAddress = delegator ? this.prefixAddress(delegator) : this.loomAddress;
        return this._dappchainDPOS.checkDelegationAsync(validatorAddress, delegatorAddress);
    }
    claimDelegatorRewardsAsync() {
        return this._dappchainDPOS.claimDelegatorRewardsAsync();
    }
    checkDelegatorRewardsAsync(owner) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const address = owner ? this.prefixAddress(owner) : this.loomAddress;
            return this._dappchainDPOS.checkDelegatorRewardsAsync(address);
        });
    }
}
exports.DPOSUserV3 = DPOSUserV3;
//# sourceMappingURL=dpos-user-v3.js.map