"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DualRPCClient = void 0;
const json_rpc_client_1 = require("./json-rpc-client");
const ws_rpc_client_1 = require("./ws-rpc-client");
const http_rpc_client_1 = require("./http-rpc-client");
/**
 * Sends JSON-RPC messages via HTTP or WebSocket, and listens to events via WebSocket.
 */
class DualRPCClient extends ws_rpc_client_1.WSRPCClient {
    /**
     * @param opts.httpUrl HTTP URL to send requests to.
     * @param opts.wsUrl WebSocket URL to connect to.
     * @param opts.protocol Primary protocol to use to send requests, defaults to HTTP.
     * @param opts.requestTimeout Number of milliseconds to wait for a network operation to complete.
     */
    constructor(opts) {
        super(opts.wsUrl, opts);
        const { protocol = json_rpc_client_1.JSONRPCProtocol.HTTP, requestTimeout, generateRequestId = this._getNextRequestId } = opts;
        this._http = new http_rpc_client_1.HTTPRPCClient(opts.httpUrl, { requestTimeout, generateRequestId });
        this._protocol = protocol;
    }
    /**
     * Sends a JSON-RPC message.
     * @param method RPC method name.
     * @param params Parameter object or array.
     * @returns A promise that will be resolved with the value of the result field (if any) in the
     *          JSON-RPC response message.
     */
    sendAsync(method, params) {
        if (this._protocol === json_rpc_client_1.JSONRPCProtocol.HTTP) {
            return this._http.sendAsync(method, params);
        }
        else {
            return this.sendAsync(method, params);
        }
    }
}
exports.DualRPCClient = DualRPCClient;
//# sourceMappingURL=dual-rpc-client.js.map