"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignedTxMiddleware = void 0;
const loom_pb_1 = require("../proto/loom_pb");
const crypto_utils_1 = require("../crypto-utils");
/**
 * Signs transactions.
 */
class SignedTxMiddleware {
    /**
     * Creates middlware that signs txs with the given key.
     * @param privateKey The private key that should be used to sign txs.
     */
    constructor(privateKey) {
        this.privateKey = privateKey;
    }
    Handle(txData) {
        const sig = (0, crypto_utils_1.sign)(txData, this.privateKey);
        const signedTx = new loom_pb_1.SignedTx();
        signedTx.setInner(txData);
        signedTx.setSignature(sig);
        signedTx.setPublicKey((0, crypto_utils_1.publicKeyFromPrivateKey)(this.privateKey));
        return Promise.resolve(signedTx.serializeBinary());
    }
}
exports.SignedTxMiddleware = SignedTxMiddleware;
//# sourceMappingURL=signed-tx-middleware.js.map